import 'package:flutter/material.dart';
import 'package:fl_chart/fl_chart.dart';

import '../../consts.dart';
import '../../utils/format_month.dart';

class BarData extends StatefulWidget {
  const BarData({super.key});

  @override
  State<StatefulWidget> createState() => _BarDataState();
}


class _BarDataState extends State<BarData> {
  static const double barWidth = 30;

  // NB should be allocated values dynamically
  /*
  selectedTile.IceStats.asMap().forEach((index, iceStats) {
    barData[index] = [iceStats.slushIce, iceStats.blackIce,
                      iceStats.snowDepth, iceStats.steelIce];
  });
   */

  // Bar items show data for 10 previous days
  static const barData = <int, List<double>>{
    0: [1.5, 3.2, 1.5, 2.2],
    1: [1.8, 4.6, 2, 3.1],
    2: [1.5, 2.1, 2.5, 1.8],
    3: [1.5, 1, 3, 2.6],
    4: [2, 2, 3.9, 2.3],
    5: [1.2, 1.2, 3.3, 2.9],
    6: [1.2, 2.3, 3.3, 3.2],
  };

  int touchedIndex = -1;

  @override
  void initState() {
    super.initState();
  }

  BarChartGroupData generateGroup(
      int x,
      double value1,
      double value2,
      double value3,
      double value4,
      ) {
    final sum = value1 + value2 + value3 + value4;
    final isTouched = touchedIndex == x;
    return BarChartGroupData(
      x: x,
      showingTooltipIndicators: isTouched ? [0] : [],
      barRods: [
        BarChartRodData(
          y: sum,
          width: barWidth,
          borderRadius: const BorderRadius.only(
            topLeft: Radius.circular(6),
            topRight: Radius.circular(6),
          ),
          rodStackItems: [
            BarChartRodStackItem(
              0,
              value1,
              const Color(0xFF13dbff),
            ),
            BarChartRodStackItem(
              value1,
              value1 + value2,
              const Color(0xFF000085),
            ),
            BarChartRodStackItem(
              value1 + value2,
              value1 + value2 + value3,
              const Color(0xFF3766E0),
            ),
            BarChartRodStackItem(
              value1 + value2 + value3,
              value1 + value2 + value3 + value4,
              Colors.white60,
            ),
          ],
        ),
      ],
    );
  }

  // _buildLegendItem renders a colored circle and text to form a legend
  Widget _buildLegendItem(Color color, String text) {
    return Row(
      children: [
        Container(
          width: 20,
          height: 20,
          decoration: BoxDecoration(
            color: color,
            shape: BoxShape.circle,
          ),
        ),
        const SizedBox(width: 8),
        Text(
          text,
          style: const TextStyle(
            fontSize: 14,
            color: Colors.white,
          ),
        ),
      ],
    );
  }


  @override
  Widget build(BuildContext context) {
      return Column(
          children: [
            SizedBox(
              width: MediaQuery.of(context).size.width, // Set the desired width
              child: BarChart(
                BarChartData(
                  alignment: BarChartAlignment.center,
                  maxY: 12,
                  minY: 0,
                  titlesData: FlTitlesData(
                    show: true,
                    bottomTitles: SideTitles(
                      showTitles: true,
                      reservedSize: 20,
                      getTextStyles: (value) => const TextStyle(color: Colors.white60),
                      getTitles: (value) {
                        // Convert bar indexes to dates
                        if (barData.isNotEmpty && value >= 0 && value < barData.length) {
                          int index = value.toInt();

                          DateTime today = DateTime.now();

                          // Subtract index from the day of the month of the current date
                          int day = today.day - (6-index);

                          String date = day.toString();
                          String month = formatMonth(today.month);

                          return '$month $date';
                        }
                        return '';
                      },
                    ),
                    leftTitles: SideTitles(
                      showTitles: true,
                      getTextStyles: (value) => const TextStyle(color: Colors.white60),
                      margin: 5,
                      reservedSize: 30,
                      interval: 2,
                    ),
                    rightTitles: SideTitles(
                      showTitles: true,
                      getTextStyles: (value) => const TextStyle(color: Colors.white60),
                      margin: 5,
                      reservedSize: 30,
                      interval: 2,
                    ),
                    topTitles: SideTitles(
                      showTitles: true,
                      getTextStyles: (value) => const TextStyle(color: Colors.white60),
                      margin: 0,
                      reservedSize: 10,
                      getTitles: (value) {
                        // Return "cm" for a specific value (e.g., 0) and empty string for others
                        return value == 0 ? 'cm' : '';
                      },
                    ),
                  ),
                  groupsSpace: 14,
                  gridData: FlGridData(
                    show: true,
                  ),
                  borderData: FlBorderData(
                    show: false,
                  ),
                  barGroups: barData.entries
                      .map(
                        (e) => generateGroup(
                      e.key,
                      e.value[0],
                      e.value[1],
                      e.value[2],
                      e.value[3],
                    ),
                  ).toList(),
                ),
              ),
            ),
            Padding( // Legend items
              padding: const EdgeInsets.only(top: 20),
              child: Center(
                child: Row(
                  mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                  children: [
                    _buildLegendItem(const Color(0xFF13dbff), "Steel ice"),
                    _buildLegendItem(const Color(0xFF000085), "Black ice"),
                    _buildLegendItem(const Color(0xFF3766E0), "Slush ice"),
                    _buildLegendItem(Colors.white60, "Snow"),
                  ],
                ),
              ),
            ),
          ],
      );
   }
}